@@ -30,18 +30,54 @@ except ImportError:
     except ImportError:
         print "Error: Neither sqlite3 nor pysqlite2 found"
         raise
+
 import os, time
 import datetime
-import hamster
-import hamster.storage
-from hamster import stuff
+import storage
+import stuff
+from shutil import copy as copyfile
 import datetime as dt
-import copy
-        
-class Storage(hamster.storage.Storage):
-    # we are saving data under $HOME/.gnome2/hamster-applet/hamster.db
+import gettext
+
+DB_FILE = 'hamster.db'
+
+class Storage(storage.Storage):
     con = None # Connection will be created on demand
 
+    def __setup(self):
+        """
+        Delayed setup so we don't do everything at the same time
+        """
+        if self.__setup.im_func.complete:
+            return
+
+        from configuration import runtime
+
+        db_file = runtime.database_file
+        db_path, _ = os.path.split(os.path.realpath(db_file))
+
+        if not os.path.exists(db_path):
+            try:
+                os.makedirs(db_path, 0744)
+            except Exception, msg:
+                print 'Error:could not create user dir (%s): %s' % (db_path, msg)
+
+        data_dir = runtime.data_dir
+
+        #check if db is here
+        if not os.path.exists(db_file):
+            print "Database not found in %s - installing default from %s!" % (db_file, data_dir)
+            copyfile(os.path.join(data_dir, DB_FILE), db_file)
+
+            #change also permissions - sometimes they are 444
+            try:
+                os.chmod(db_file, 0664)
+            except Exception, msg:
+                print 'Error:could not change mode on %s!' % (db_file)
+        self.__setup.im_func.complete = True
+        self.run_fixtures()
+    __setup.complete = False
+
     def __get_category_list(self):
         return self.fetchall("SELECT * FROM categories ORDER BY category_order")
 
@@ -207,8 +243,10 @@ class Storage(hamster.storage.Storage):
         last = self.fetchone(query, (_("Unsorted"), dt.date.today()))
         if not last:
             #try yesterday if there is nothing today
-            last = self.fetchone(query, (_("Unsorted"),
-                                         dt.date.today() - dt.timedelta(days=1)))
+            last = self.fetchone(query,
+                                 (# unsorted category
+                                  _("Unsorted"),
+                                  dt.date.today() - dt.timedelta(days=1)))
 
         if last and last["end_time"]: #will consider as last only if it is going on
            last = None
@@ -226,22 +264,51 @@ class Storage(hamster.storage.Storage):
             """
             self.execute(query, (end_time, fact['id']))
 
-    def __solve_overlaps(self, start_time, end_time):
-        """finds facts that happen in given interval and shifts them to
-        make room for new fact"""
-        
-        # in case of missing end_time, treat is as ongoing task, but only
-        # if that's not too far in past
-        if not end_time and (dt.datetime.now() - start_time <= dt.timedelta(days=1)):
-            end_time = dt.datetime.now()
-            
-        
-        # activities that we are overlapping.
-        # second OR clause is for elimination - |new fact--|---old-fact--|--new fact|
+    def __squeeze_in(self, start_time):
+        # tries to put task in the given date
+        # if there are conflicts, we will only truncate the ongoing task
+        # and replace it's end part with our activity
+
+        # we are checking if our start time is in the middle of anything
+        # or maybe there is something after us - so we know to adjust end time
         query = """
                    SELECT a.*, b.name
                      FROM facts a
                 LEFT JOIN activities b on b.id = a.activity_id
+                    WHERE ((start_time < ? and end_time > ?) or start_time > ?)
+                 ORDER BY start_time
+                    LIMIT 1
+                """
+        fact = self.fetchone(query, (start_time, start_time, start_time))
+
+        end_time = None        
+
+        if fact:
+            if fact["start_time"] < start_time and fact["end_time"]:
+                #we are in middle of a task
+                self.execute("UPDATE facts SET end_time=? WHERE id=?",
+                             (start_time, fact["id"]))
+                end_time = fact["end_time"]
+            else: #otherwise we have found a task that is after us
+                end_time = fact["start_time"]
+
+        return end_time
+        
+    def __solve_overlaps(self, start_time, end_time):
+        """finds facts that happen in given interval and shifts them to
+        make room for new fact"""
+
+        # this function is destructive - can't go with a wildcard
+        if not end_time or not start_time: #this function is destructive
+            return
+        
+        # activities that we are overlapping.
+        # second OR clause is for elimination - |new fact--|---old-fact--|--new fact|
+        query = """
+                   SELECT a.*, b.name, c.name as category
+                     FROM facts a
+                LEFT JOIN activities b on b.id = a.activity_id
+                LEFT JOIN categories c on b.category_id = c.id
                     WHERE ((start_time < ? and end_time > ?)
                            OR (start_time < ? and end_time > ?))
                            
@@ -261,7 +328,12 @@ class Storage(hamster.storage.Storage):
                 self.execute("""UPDATE facts
                                    SET end_time = ?
                                  WHERE id = ?""", (start_time, fact["id"]))
-                self.__add_fact(fact["name"], end_time, fact["end_time"])
+                fact_name = fact["name"]
+                self.__add_fact(fact["name"],
+                                end_time,
+                                fact["end_time"],
+                                fact["category"],
+                                fact["description"])
 
             #eliminate
             elif fact["end_time"] and \
@@ -283,10 +355,24 @@ class Storage(hamster.storage.Storage):
                              (start_time, fact["id"]))
 
 
-    def __add_fact(self, activity_name, start_time = None, end_time = None):
+    def __add_fact(self, activity_name, start_time = None,
+                     end_time = None, category_name = None, description = None):
         activity = stuff.parse_activity_input(activity_name)
+        
+        if category_name:
+            activity.category_name = category_name
+        if description:
+            activity.description = description
+        
         start_time = activity.start_time or start_time or datetime.datetime.now()
+        
+        if start_time > datetime.datetime.now():
+            return None #no facts in future, please
+        
+        start_time = start_time.replace(microsecond = 0)
         end_time = activity.end_time or end_time
+        if end_time:
+            end_time = end_time.replace(microsecond = 0)
 
             
         # now check if maybe there is also a category
@@ -314,13 +400,14 @@ class Storage(hamster.storage.Storage):
                    and last_activity['activity_id'] == activity_id:
                     return last_activity
                 
+                #if duration is less than a minute - it must have been a mistake
                 if not activity.description \
                    and not last_activity["description"] \
                    and 60 >= (start_time - last_activity['start_time']).seconds >= 0:
                     self.__remove_fact(last_activity['id'])
                     start_time = last_activity['start_time']
                 else:
-                    #stop 
+                    #otherwise stop 
                     update = """
                                UPDATE facts
                                   SET end_time = ?
@@ -329,8 +416,11 @@ class Storage(hamster.storage.Storage):
                     self.execute(update, (start_time, last_activity["id"]))
 
 
-        #done with the current activity, now we can solve overlaps
-        self.__solve_overlaps(start_time, end_time)
+        # done with the current activity, now we can solve overlaps
+        if not end_time:
+            end_time = self.__squeeze_in(start_time)
+        else:
+            self.__solve_overlaps(start_time, end_time)
 
 
         # finally add the new entry
@@ -345,7 +435,7 @@ class Storage(hamster.storage.Storage):
         return self.__get_fact(fact_id)
 
 
-    def __get_facts(self, date, end_date = None):
+    def __get_facts(self, date, end_date = None, category_id = None):
         query = """
                    SELECT a.id AS id,
                           a.start_time AS start_time,
@@ -355,57 +445,75 @@ class Storage(hamster.storage.Storage):
                           coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id
                      FROM facts a
                 LEFT JOIN activities b ON a.activity_id = b.id
-                LEFT JOIN categories c on b.category_id = c.id
-                    WHERE (date(a.start_time) >= ? and date(a.start_time) <= ?)
-                       OR (date(a.end_time) >= ? and date(a.end_time) <= ?)
-                 ORDER BY a.start_time
+                LEFT JOIN categories c ON b.category_id = c.id
+                    WHERE (a.end_time >= ? OR a.end_time IS NULL) AND a.start_time <= ?
         """
-        end_date = end_date or date
         
-        facts = self.fetchall(query, (_("Unsorted"), date, end_date, date, end_date))
+        if category_id and isinstance(category_id, int):
+            query += " and b.category_id = %d" % category_id
+        elif category_id and isinstance(category_id, list):
+            query += " and b.category_id IN (%s)" % (",".join([str(id) for id in category_id]))
+
+        query += " ORDER BY a.start_time"
+        end_date = end_date or date
+
+        #FIXME: add preference to set that
+        split_time = dt.time(5, 30)
+        datetime_from = dt.datetime.combine(date, split_time)
+        datetime_to = dt.datetime.combine(end_date, split_time) + dt.timedelta(days = 1)
+        
+        facts = self.fetchall(query, (_("Unsorted"), datetime_from, datetime_to))
         res = []
 
         today = dt.date.today()
-        yesterday = dt.date.today() - dt.timedelta(days=1)
+        yesterday = dt.date.today() - dt.timedelta(days = 1)
         now = dt.datetime.now()
         
-
         # fetch last activity here - we will be looking it up for comparisons
         last_activity = None
-        if date >= today >= end_date or abs(date-today).days < 2 \
-                                   or abs(end_date-today).days < 2:
+        if date >= today >= end_date or abs(date - today).days < 2 \
+                                   or abs(end_date - today).days < 2:
             last_activity = self.__get_last_activity()
 
-        # deal with late-night workers!
-        if not facts and date == today:
-            # no facts today?! let's get last fact of yesterday
-            if last_activity:
-                # last fact has not finished, we think that it is still ongoing
-                f = dict(
-                    id = last_activity["id"],
-                    start_time = dt.datetime.combine(date, dt.time(0,0)),
-                    end_time = now,
-                    description = last_activity["description"],
-                    name = last_activity["name"],
-                    activity_id = last_activity["activity_id"],
-                    category = last_activity["category"],
-                    category_id = last_activity["category_id"],
-                    delta = now - dt.datetime.combine(date, dt.time(0,0))
-                )
-            
-                return [f]
-
-
         for fact in facts:
-            fact_start_date = fact["start_time"].date()
+            # heuristics to assign tasks to proper days
             if fact["end_time"]:
-                fact_end_date = fact["end_time"].date()
+                fact_end_time = fact["end_time"]
+            elif (today - fact["start_time"].date()) <= dt.timedelta(days=1):
+                fact_end_time = now
             else:
-                fact_end_date = None
-                                   
+                fact_end_time = fact["start_time"].replace(hour=23, minute=59)
+
+            fact_start_date = fact["start_time"].date() \
+                - dt.timedelta(1 if fact["start_time"].time() < split_time else 0)
+            fact_end_date = fact_end_time.date() \
+                - dt.timedelta(1 if fact_end_time.time() < split_time else 0)
+            fact_date_span = fact_end_date - fact_start_date
+
+            # check if the task spans across two dates
+            if fact_date_span.days == 1:
+                datetime_split = dt.datetime.combine(fact_end_date, split_time)
+                start_date_duration = datetime_split - fact["start_time"]
+                end_date_duration = fact_end_time - datetime_split
+                if start_date_duration > end_date_duration:
+                    # most of the task was done during the previous day
+                    fact_date = fact_start_date
+                else:
+                    fact_date = fact_end_date
+            else:
+                # either doesn't span or more than 24 hrs tracked
+                # (in which case we give up)
+                fact_date = fact_start_date
+
+            if fact_date < date or fact_date > end_date:
+                # due to spanning we've jumped outside of given period
+                continue
+
             f = dict(
                 id = fact["id"],
                 start_time = fact["start_time"],
+                date = fact_date,
+                delta = fact_end_time - fact["start_time"],
                 end_time = fact["end_time"],
                 description = fact["description"],
                 name = fact["name"],
@@ -413,46 +521,7 @@ class Storage(hamster.storage.Storage):
                 category = fact["category"],
                 category_id = fact["category_id"]
             )
-            
-            if not fact_end_date:
-                if fact_start_date == today and fact["start_time"] < now:
-                    # today, present
-                    f["delta"] = now -  fact["start_time"]
-                elif last_activity and fact_start_date == yesterday and f["id"] == last_activity["id"]:
-                    # last fact and it is in yesterday - split it!
-                    #first yesterday until midnight
-                    f["end_time"] = dt.datetime.combine(today, dt.time(0,0))
-                    f["delta"] = f["end_time"] - f["start_time"]
-                    res.append(f)
-                    
-                    #and now today until now
-                    f = copy.copy(f)
-                    f["start_time"] = dt.datetime.combine(today, dt.time(0, 0))
-                    f["end_time"] = now
-                    f["delta"] = f["end_time"] - f["start_time"]
-                else:
-                    f["delta"] = None
-
-                res.append(f)
-            elif fact_start_date != fact_end_date:
-                # check if maybe we have to split activity in two
-                if date <= fact["start_time"].date()  <= end_date:
-                    start_fact = copy.copy(f)
-                    start_fact["end_time"] = dt.datetime.combine(f["end_time"],
-                                                                 dt.time(0, 0))
-                    start_fact["delta"] = start_fact["end_time"] - start_fact["start_time"]
-                    res.append(start_fact)
-
-                if date <= fact["end_time"].date()  <= end_date:
-                    end_fact = copy.copy(f)
-                    end_fact["start_time"] = dt.datetime.combine(f["end_time"],
-                                                                 dt.time(0, 0))
-                    end_fact["delta"] = end_fact["end_time"] - end_fact["start_time"]
-                    res.append(end_fact)
-            else:
-                #else is we have end date and it is the same date
-                f["delta"] = fact["end_time"] - fact["start_time"]
-                res.append(f)
+            res.append(f)
 
         return res
 
@@ -468,21 +537,6 @@ class Storage(hamster.storage.Storage):
         """
         return self.fetchall(query, (_("Unsorted"), ))
 
-    def __get_interval_activity_ids(self, date, end_date = None):
-        """returns activities used in the specified interval"""
-        query = """
-                   SELECT a.name, coalesce(b.name, ?) as category_name
-                     FROM activities a
-                LEFT JOIN categories b on b.id = a.category_id
-                    WHERE a.id in (SELECT activity_id from facts
-                                    WHERE (date(start_time) >= ? and date(start_time) <= ?)
-                                       OR (date(end_time) >= ? and date(end_time) <= ?))
-        """
-        end_date = end_date or date        
-
-        return self.fetchall(query, (_("Unsorted"), date, end_date, date, end_date))
-        
-    
     def __remove_fact(self, fact_id):
         query = """
                    DELETE FROM facts
@@ -607,8 +661,10 @@ class Storage(hamster.storage.Storage):
 
     """ Here be dragons (lame connection/cursor wrappers) """
     def get_connection(self):
+        from configuration import runtime
         if self.con is None:
-            self.con = sqlite.connect(hamster.HAMSTER_DB, detect_types=sqlite.PARSE_DECLTYPES|sqlite.PARSE_COLNAMES)
+            db_file = runtime.database_file
+            self.con = sqlite.connect(db_file, detect_types=sqlite.PARSE_DECLTYPES|sqlite.PARSE_COLNAMES)
             self.con.row_factory = sqlite.Row
 
         return self.con
@@ -616,10 +672,13 @@ class Storage(hamster.storage.Storage):
     connection = property(get_connection, None)
 
     def fetchall(self, query, params = None):
+        from configuration import runtime
+        self.__setup()
+
         con = self.connection
         cur = con.cursor()
 
-        if hamster.trace_sql:
+        if runtime.trace_sql:
             print query, params
 
         if params:
@@ -640,8 +699,13 @@ class Storage(hamster.storage.Storage):
             return None
 
     def execute(self, statement, params = ()):
-        """execute sql statement. optionally you can give multiple statements
-        to save on cursor creation and closure"""
+        """
+        execute sql statement. optionally you can give multiple statements
+        to save on cursor creation and closure
+        """
+        from configuration import runtime
+        self.__setup()
+
         con = self.connection
         cur = con.cursor()
         
@@ -651,7 +715,7 @@ class Storage(hamster.storage.Storage):
             
         if isinstance(statement, list):
             for i in range(len(statement)):
-                if hamster.trace_sql:
+                if runtime.trace_sql:
                     print statement[i], params[i]
          
                 res = cur.execute(statement[i], params[i])
@@ -667,7 +731,7 @@ class Storage(hamster.storage.Storage):
                                      _("Super secret project X"),
                                      _("World domination")]}
         
-        nonwork_category = {"name": _("Day to day"),
+        nonwork_category = {"name": _("Day-to-day"),
                             "entries": [_("Lunch"),
                                         _("Watering flowers"),
                                         _("Doing handstands")]}
@@ -860,7 +924,4 @@ class Storage(hamster.storage.Storage):
             nonwork_cat_id = self.__add_category(nonwork_category["name"])
             for entry in nonwork_category["entries"]:
                 self.__add_activity(entry, nonwork_cat_id)
-        
-        
-        
 
