@@ -39,6 +39,7 @@ from gettext import ngettext
 import datetime as dt
 import calendar
 import time
+from hamster.i18n import C_
 
 class ReportChooserDialog(gtk.Dialog):
     __gsignals__ = {
@@ -68,7 +69,7 @@ class ReportChooserDialog(gtk.Dialog):
         self.dialog.add_filter(filter)
 
         filter = gtk.FileFilter()
-        filter.set_name(_("Tab Separated Values (TSV)"))
+        filter.set_name(_("Tab-Separated Values (TSV)"))
         filter.add_mime_type("text/plain")
         filter.add_pattern("*.tsv")
         filter.add_pattern("*.txt")
@@ -116,7 +117,7 @@ class ReportChooserDialog(gtk.Dialog):
         self.end_date.set_date(end_date)
         
         #add unsorted category
-        button_all = gtk.CheckButton(_("All").encode("utf-8"))
+        button_all = gtk.CheckButton(C_("categories", "All").encode("utf-8"))
         button_all.value = None
         button_all.set_active(True)
         
@@ -390,6 +391,8 @@ class StatsViewer(object):
         self.week_view = self.get_widget("week")
         self.month_view = self.get_widget("month")
         self.month_view.set_group(self.week_view)
+        self.day_view = self.get_widget("day")
+        self.day_view.set_group(self.week_view)
         
         #initiate the form in the week view
         self.week_view.set_active(True)
@@ -429,7 +432,9 @@ class StatsViewer(object):
                 self.year = year
                 self.connect("clicked", on_clicked)
         
-        all_button = YearButton(_("All"), None, self.on_year_changed)
+        all_button = YearButton(C_("years", "All").encode("utf-8"),
+                                None,
+                                self.on_year_changed)
         year_box.pack_start(all_button)
         self.bubbling = True # TODO figure out how to properly work with togglebuttons as radiobuttons
         all_button.set_active(True)
@@ -519,7 +524,7 @@ class StatsViewer(object):
                 label.set_text(_("""There is no data to generate statistics yet.
 A week of usage would be nice!"""))
             else:
-                label.set_text(_("Still collecting data - check back after a week has passed!"))
+                label.set_text(_("Still collecting data — check back after a week has passed!"))
 
             label.show()
             return
@@ -646,13 +651,15 @@ A week of usage would be nice!"""))
 
         # first record        
         if not year:
-            #date format for case when year has not been selected
-            first_date = _("%(first_b)s %(first_d)s, %(first_Y)s") % \
-                               stuff.dateDict(facts[0]["start_time"], "first_")
+            # date format for the first record if the year has not been selected
+            # Using python datetime formatting syntax. See:
+            # http://docs.python.org/library/time.html#time.strftime
+            first_date = facts[0]["start_time"].strftime(C_("first record", "%b %d, %Y"))
         else:
-            #date format when year has been selected
-            first_date = _("%(first_b)s %(first_d)s") % \
-                               stuff.dateDict(facts[0]["start_time"], "first_")
+            # date of first record when year has been selected
+            # Using python datetime formatting syntax. See:
+            # http://docs.python.org/library/time.html#time.strftime
+            first_date = facts[0]["start_time"].strftime(C_("first record", "%(b)s %(d)s"))
 
         summary += _("First activity was recorded on %s.") % \
                                                      ("<b>%s</b>" % first_date)
@@ -686,18 +693,24 @@ A week of usage would be nice!"""))
             if not max_fact or fact["delta"] > max_fact["delta"]:
                 max_fact = fact
 
-        datedict = stuff.dateDict(max_fact["start_time"], "max_")
+        longest_date = max_fact["start_time"].strftime(
+            # How the date of the longest activity should be displayed in statistics
+            # Using python datetime formatting syntax. See:
+            # http://docs.python.org/library/time.html#time.strftime
+            C_("date of the longest activity", "%b %d, %Y"))
+        
         num_hours = max_fact["delta"].seconds / 60 / 60.0 + max_fact["delta"].days * 24
-        datedict["hours"] = "<b>%.1f</b>" % (num_hours)
-
+        hours = "<b>%.1f</b>" % (num_hours)
+        
         summary += "\n" + ngettext("Longest continuous work happened on \
-%(max_b)s %(max_d)s, %(max_Y)s and was %(hours)s hour.",
+%(date)s and was %(hours)s hour.",
                                   "Longest continuous work happened on \
-%(max_b)s %(max_d)s, %(max_Y)s and was %(hours)s hours.",
-                                  int(num_hours)) % datedict
+%(date)s and was %(hours)s hours.",
+                                  int(num_hours)) % {"date": longest_date,
+                                                     "hours": hours}
 
         # total records (in selected scope)
-        summary += " " + ngettext("There are %s record.",
+        summary += " " + ngettext("There is %s record.",
                                   "There are %s records.",
                                   len(facts)) % ("<b>%d</b>" % len(facts))
 
@@ -793,6 +806,13 @@ than 15 minutes you seem to be a busy bee." % ("<b>%d</b>" % short_percent))
         
         self.fact_tree.set_model(self.fact_store)
     
+    def on_graph_frame_size_allocate(self, widget, new_size):
+        w = min(new_size.width / 4, 200)
+        
+        self.activity_chart.legend_width = w
+        self.category_chart.legend_width = w
+        self.get_widget("totals_by_category").set_size_request(w + 40, -1)
+    
     def fill_tree(self, facts):
         day_dict = {}
         for day, facts in groupby(facts, lambda fact: fact["date"]):
@@ -802,9 +822,10 @@ than 15 minutes you seem to be a busy bee." % ("<b>%d</b>" % short_percent))
         for i in range((self.end_date - self.start_date).days  + 1):
             current_date = self.start_date + dt.timedelta(i)
             
-            # date format in overview window fact listing
-            # prefix is "o_",letter after prefix is regular python format. you can use all of them
-            fact_date = _("%(o_A)s, %(o_b)s %(o_d)s") %  stuff.dateDict(current_date, "o_")
+            # Date format for the label in overview window fact listing
+            # Using python datetime formatting syntax. See:
+            # http://docs.python.org/library/time.html#time.strftime
+            fact_date = current_date.strftime(C_("overview list", "%A, %b %d"))
             
             day_total = dt.timedelta()
             for fact in day_dict.get(current_date, []):
@@ -864,7 +885,10 @@ than 15 minutes you seem to be a busy bee." % ("<b>%d</b>" % short_percent))
         if (self.end_date - self.start_date).days < 20:
             day_keys = [day.strftime("%a") for day in all_days]
         else:
-            day_keys = [_("%(m_b)s %(m_d)s") %  stuff.dateDict(day, "m_")
+            # date format used in the overview graph when month view is selected
+            # Using python datetime formatting syntax. See:
+            # http://docs.python.org/library/time.html#time.strftime
+            day_keys = [day.strftime(C_("overview graph", "%b %d"))
                                                             for day in all_days]
 
         self.day_chart.plot(day_keys, res, stack_keys = all_categories)
@@ -888,29 +912,44 @@ than 15 minutes you seem to be a busy bee." % ("<b>%d</b>" % short_percent))
         
 
     def set_title(self):
-        dates_dict = stuff.dateDict(self.start_date, "start_")
-        dates_dict.update(stuff.dateDict(self.end_date, "end_"))
-        
-        if self.start_date.year != self.end_date.year:
-            # overview label if start and end years don't match
-            # letter after prefixes (start_, end_) is the one of
-            # standard python date formatting ones- you can use all of them
-            overview_label = _(u"Overview for %(start_B)s %(start_d)s, %(start_Y)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
-        elif self.start_date.month != self.end_date.month:
-            # overview label if start and end month do not match
-            # letter after prefixes (start_, end_) is the one of
-            # standard python date formatting ones- you can use all of them
-            overview_label = _(u"Overview for %(start_B)s %(start_d)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
+        if self.day_view.get_active():
+            # date format for overview label when only single day is visible
+            # Using python datetime formatting syntax. See:
+            # http://docs.python.org/library/time.html#time.strftime
+            start_date_str = self.view_date.strftime(C_("single day overview",
+                                                        "%B %d, %Y"))
+            # Overview label if looking on single day
+            overview_label = _(u"Overview for %(date)s") % \
+                                                      ({"date": start_date_str})
         else:
-            # overview label for interval in same month
-            # letter after prefixes (start_, end_) is the one of
-            # standard python date formatting ones- you can use all of them
-            overview_label = _(u"Overview for %(start_B)s %(start_d)s – %(end_d)s, %(end_Y)s") % dates_dict
+            dates_dict = stuff.dateDict(self.start_date, "start_")
+            dates_dict.update(stuff.dateDict(self.end_date, "end_"))
+            
+            if self.start_date.year != self.end_date.year:
+                # overview label if start and end years don't match
+                # letter after prefixes (start_, end_) is the one of
+                # standard python date formatting ones- you can use all of them
+                # see http://docs.python.org/library/time.html#time.strftime
+                overview_label = _(u"Overview for %(start_B)s %(start_d)s, %(start_Y)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
+            elif self.start_date.month != self.end_date.month:
+                # overview label if start and end month do not match
+                # letter after prefixes (start_, end_) is the one of
+                # standard python date formatting ones- you can use all of them
+                # see http://docs.python.org/library/time.html#time.strftime
+                overview_label = _(u"Overview for %(start_B)s %(start_d)s – %(end_B)s %(end_d)s, %(end_Y)s") % dates_dict
+            else:
+                # overview label for interval in same month
+                # letter after prefixes (start_, end_) is the one of
+                # standard python date formatting ones- you can use all of them
+                # see http://docs.python.org/library/time.html#time.strftime
+                overview_label = _(u"Overview for %(start_B)s %(start_d)s – %(end_d)s, %(end_Y)s") % dates_dict
 
         if self.week_view.get_active():
             dayview_caption = _("Week")
-        else:
+        elif self.month_view.get_active():
             dayview_caption = _("Month")
+        else:
+            dayview_caption = _("Day")
         
         self.get_widget("overview_label").set_markup("<b>%s</b>" % overview_label)
         self.get_widget("dayview_caption").set_markup("%s" % (dayview_caption))
@@ -919,17 +958,22 @@ than 15 minutes you seem to be a busy bee." % ("<b>%d</b>" % short_percent))
     def do_graph(self):
         self.set_title()
         
-        facts = runtime.storage.get_facts(self.start_date, self.end_date)
+        if self.day_view.get_active():
+            facts = runtime.storage.get_facts(self.view_date)
+        else:
+            facts = runtime.storage.get_facts(self.start_date, self.end_date)
+
 
         self.get_widget("report_button").set_sensitive(len(facts) > 0)
         self.fact_store.clear()
         
+        self.fill_tree(facts)
+
         if not facts:
             self.get_widget("graphs").hide()
             self.get_widget("no_data_label").show()
             return 
 
-        self.fill_tree(facts)
 
         self.get_widget("no_data_label").hide()
         self.get_widget("graphs").show()
@@ -964,29 +1008,42 @@ than 15 minutes you seem to be a busy bee." % ("<b>%d</b>" % short_percent))
         self.stats(button.year)
 
     def on_prev_clicked(self, button):
-        if self.week_view.get_active():
-            self.start_date -= dt.timedelta(7)
-            self.end_date -= dt.timedelta(7)
-        
-        elif self.month_view.get_active():
-            self.end_date = self.start_date - dt.timedelta(1)
-            first_weekday, days_in_month = calendar.monthrange(self.end_date.year, self.end_date.month)
-            self.start_date = self.end_date - dt.timedelta(days_in_month - 1)
+        if self.day_view.get_active():
+            self.view_date -= dt.timedelta(1)
+            if self.view_date < self.start_date:
+                self.start_date -= dt.timedelta(7)
+                self.end_date -= dt.timedelta(7)
+        else:
+            if self.week_view.get_active():
+                self.start_date -= dt.timedelta(7)
+                self.end_date -= dt.timedelta(7)
+            
+            elif self.month_view.get_active():
+                self.end_date = self.start_date - dt.timedelta(1)
+                first_weekday, days_in_month = calendar.monthrange(self.end_date.year, self.end_date.month)
+                self.start_date = self.end_date - dt.timedelta(days_in_month - 1)
+
+            self.view_date = self.start_date
 
-        self.view_date = self.start_date        
         self.do_graph()
 
     def on_next_clicked(self, button):
-        if self.week_view.get_active():
-            self.start_date += dt.timedelta(7)
-            self.end_date += dt.timedelta(7)
+        if self.day_view.get_active():
+            self.view_date += dt.timedelta(1)
+            if self.view_date > self.end_date:
+                self.start_date += dt.timedelta(7)
+                self.end_date += dt.timedelta(7)
+        else:
+            if self.week_view.get_active():
+                self.start_date += dt.timedelta(7)
+                self.end_date += dt.timedelta(7)        
+            elif self.month_view.get_active():
+                self.start_date = self.end_date + dt.timedelta(1)
+                first_weekday, days_in_month = calendar.monthrange(self.start_date.year, self.start_date.month)
+                self.end_date = self.start_date + dt.timedelta(days_in_month - 1)
         
-        elif self.month_view.get_active():
-            self.start_date = self.end_date + dt.timedelta(1)
-            first_weekday, days_in_month = calendar.monthrange(self.start_date.year, self.start_date.month)
-            self.end_date = self.start_date + dt.timedelta(days_in_month - 1)
-        
-        self.view_date = self.start_date
+            self.view_date = self.start_date
+
         self.do_graph()
     
     def on_home_clicked(self, button):
@@ -1004,8 +1061,10 @@ than 15 minutes you seem to be a busy bee." % ("<b>%d</b>" % short_percent))
         self.do_graph()
         
     def on_day_toggled(self, button):
-        self.start_date = self.view_date
-        self.end_date = self.view_date
+        self.start_date = self.view_date - dt.timedelta(self.view_date.weekday() + 1)
+        self.start_date = self.start_date + dt.timedelta(stuff.locale_first_weekday())
+
+        self.end_date = self.start_date + dt.timedelta(6)
         self.do_graph()
 
     def on_week_toggled(self, button):
@@ -1187,7 +1246,7 @@ than 15 minutes you seem to be a busy bee." % ("<b>%d</b>" % short_percent))
         chooser.add_filter(filter)
 
         filter = gtk.FileFilter()
-        filter.set_name(_("Tab Separated Values (TSV)"))
+        filter.set_name(_("Tab-Separated Values (TSV)"))
         filter.add_mime_type("text/plain")
         filter.add_pattern("*.tsv")
         filter.add_pattern("*.txt")
